/* {{{ Copyright etc. */

/*  ip - FMRI image preprocessing

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

/* }}} */
/* {{{ includes */

#include "libss/libss.h"
#include "libss/libavw.h"

/* }}} */
/* {{{ usage */

void usage(char *progname)
{
  printf("Usage: %s <input> <output> <%%threshold> [options]\n\n",progname);
  printf("-s <sigma> : spatial lowpass Gaussian linear filtering (sigma in mm, not voxels)\n");
  printf("-m <mask_filename> : create binary mask, apply to fmri data and save mask to file\n");
  printf("-i <target mean> : global intensity normalisation (separate intensity rescaling for each 3D volume)\n");
  printf("-I <target mean> : grand mean intensity normalisation (one intensity scaling for the whole 4D data set)\n");
  printf("-t <hp_sigma> <lp_sigma> : Bandpass temporal filtering; nonlinear highpass and Gaussian linear lowpass (with sigmas in volumes, not seconds); set either sigma<0 to skip that filter\n\n");
  exit(1);
}

/* }}} */
/* {{{ main(argc, argv) */

int main(argc, argv)
  int   argc;
  char  *argv [];
{
/* {{{ vars */

int          i;
double       percent_thresh;
image_struct im, mask;

/* }}} */

  /* {{{ process initial arguments and find threshold */

if (argc<4)
     usage(argv[0]);

avw_read(argv[1],&im);

percent_thresh = atof(argv[3]);

if ( (percent_thresh<0) || (percent_thresh>100) )
{
  printf("<%%threshold> must lie between 0 and 100\n");
  usage(argv[0]);
}

percent_thresh/=100.0;

find_thresholds (&im, percent_thresh);
/*printf("thresholds: 0=%f 2=%f t=%f 98=%f 100=%f\n",im.min,im.thresh2,im.thresh,im.thresh98,im.max);*/

/* }}} */
  /* {{{ process options */

for (i=4; i<argc; i++) {
  if (!strcmp(argv[i], "-s"))
    /* {{{ spatial filtering */

{
  double sigma;

  i++;
  if (i==argc) usage(argv[0]);
  sigma = atof(argv[i]);

  if (sigma<0)
    {
      printf("Spatial sigma must be >=0\n");
      usage(argv[0]);
    }

  spatfilt(im,sigma);
}

/* }}} */
  else if (!strcmp(argv[i], "-m"))
    /* {{{ threshold and create mask */

{
  i++;
  if (i==argc) usage(argv[0]);

  mask=im;
  mask.t=1;
  mask.dt=DT_UNSIGNED_CHAR;
  mask.i=malloc(sizeof(unsigned char)*mask.x*mask.y*mask.z);
  mask.min=0;
  mask.max=1;

  fmrimask(im,mask);

  avw_write(argv[i],mask);
}

/* }}} */
  else if ( (!strcmp(argv[i], "-i")) || (!strcmp(argv[i], "-I")) )
    /* {{{ intensity normalisation */

{
  double target_mean;
  int grand_mean=0;

  if (!strcmp(argv[i], "-I"))
    grand_mean=1;

  i++;
  if (i==argc) usage(argv[0]);
  target_mean = atof(argv[i]);

  intnorm(im,grand_mean,target_mean);
}

/* }}} */
  else if (!strcmp(argv[i], "-t"))
    /* {{{ temporal filtering */

{
  double hp_sigma, lp_sigma;

  i++;
  if (i==argc) usage(argv[0]);
  hp_sigma = atof(argv[i]);

  i++;
  if (i==argc) usage(argv[0]);
  lp_sigma = atof(argv[i]);

  tempfilt(im,hp_sigma,lp_sigma);
}

/* }}} */
  else
    usage(argv[0]);
}

/* }}} */
  /* {{{ output and exit */

avw_write(argv[2],im);

printf("THRESHOLD %f\n",im.thresh);

return(0);

/* }}} */
}

/* }}} */
