/* {{{ Copyright etc. */

/*  overlay.c - combine two images for colour overlay

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"
#include "libss/libtessa.h"

void usage(void);

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("Usage: overlay <colour_type> <output_type> <bg> <bgmin> <bgmax> <s1> <s1min> <s1max> [s2 s2min s2max] <output>\n");
  printf("colour_type: 0=solid 1=transparent colours\n");
  printf("output_type: 0=floating point (32 bit real) 1=integer (16 bit signed integer)\n");
  exit(1);
}

/* }}} */
/* {{{ main */

#define NS 2
#define LATITUDE 0.00001

int main(argc, argv)
  int   argc;
  char  *argv [];
{
  /* {{{ vars */

FDT *bg, bgmin, bgmax, *s[NS], smin[NS], smax[NS];
image_struct im;
int size, i, ns=1, output_type, colour_type;
short int *ibg;
float hrange, A, B;

/* }}} */

  /* {{{ process arguments and input images */

if (argc<10)
  usage();

colour_type=atoi(argv[1]);
output_type=atoi(argv[2]);

avw_read(argv[3],&im); bg=im.i;
size=im.x*im.y*im.z;
bgmin=atof(argv[4]);
bgmax=atof(argv[5]);

avw_read(argv[6],&im); s[0]=im.i;
if (size!=im.x*im.y*im.z)
{
  printf("Input images aren't all the same size!\n");
  exit(1);
}
smin[0]=atof(argv[7]);
smax[0]=atof(argv[8]);

if (argc>10)
{
  ns=2;
  avw_read(argv[9],&im); s[1]=im.i;
  if (size!=im.x*im.y*im.z)
    {
      printf("Input images aren't all the same size!\n");
      exit(1);
    }
  smin[1]=atof(argv[10]);
  smax[1]=atof(argv[11]);
}

/* }}} */
  /* {{{ setup ranges */

if ( output_type == 0 ) /* output floats or ints? */
{
  A = ( 13*smin[0] - 7*smax[0] ) / 6;
  B = ( 20*smin[0] + A ) / 21;
}
else
{
  A = 0;
  B = 32000.0/( 1 + ( ns * ns ) );
}

hrange = B - A;

im.min=A;
im.max=A + ( hrange * ( 1 + ( ns * ns ) ) );

/* }}} */
  /* {{{ COMMENT main processing before allowing stats>maxstats */

#ifdef FoldingComment

for(i=0; i<size; i++)
{
  float newbg = ( MIN(MAX(bg[i],bgmin),bgmax) - bgmin ) / (bgmax-bgmin);   /* limited, rescaled bg */
  float news0 = (s[0][i]-smin[0]) / (smax[0]-smin[0]);                     /* rescaled s[0] */
  int   mask0 = (s[0][i]>=smin[0]-LATITUDE) * (s[0][i]<=smax[0]+LATITUDE); /* s[0] mask */

  if ( ns == 1 )
  {
    if ( colour_type == 0 ) /* solid colours */
      bg[i] = A + newbg*0.95*hrange*(1-mask0) + ( news0*0.9*hrange + 1.05*hrange ) * mask0;
    else                    /* transparent colours */
      bg[i] = A + newbg*0.95*hrange*(1-mask0) +
	( ((int)(news0*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.2 + 1.05*hrange
        ) * mask0;
  } 
  else
  {
    float news1 = (s[1][i]-smin[1]) / (smax[1]-smin[1]);                     /* rescaled s[1] */
    int   mask1 = (s[1][i]>=smin[1]-LATITUDE) * (s[1][i]<=smax[1]+LATITUDE); /* s1 mask */

    if ( colour_type == 0 ) /* solid colours */
      bg[i] = A + newbg*hrange*0.95*(1-mask0)*(1-mask1) +
	( news0*hrange*0.9 + hrange*1.05 ) * mask0 +
	( news1*hrange*0.9 + hrange*2.05 ) * mask1;
    else
      bg[i] = A + newbg*hrange*0.95*(1-mask0)*(1-mask1) +
	( ((int)(news0*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.2 + 1.05*hrange
        ) * mask0*(1-mask1) +
	( ((int)(news1*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.2 + 2.05*hrange
        ) * mask1*(1-mask0) +
	( ((int)((news0+news1)*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.4 + 3.05*hrange
        ) * mask0*mask1;
  }
}

#endif

/* }}} */
  /* {{{ main processing */

for(i=0; i<size; i++)
{
  float newbg = ( MIN(MAX(bg[i],bgmin),bgmax) - bgmin ) / (bgmax-bgmin);   /* limited, rescaled bg */
  float news0 = (MIN(s[0][i],smax[0])-smin[0]) / (smax[0]-smin[0]);        /* rescaled s[0] */
  int   mask0 = s[0][i]>=smin[0]-LATITUDE;                                 /* s[0] mask */

  if ( ns == 1 )
  {
    if ( colour_type == 0 ) /* solid colours */
      bg[i] = A + newbg*0.95*hrange*(1-mask0) + ( news0*0.9*hrange + 1.05*hrange ) * mask0;
    else                    /* transparent colours */
      bg[i] = A + newbg*0.95*hrange*(1-mask0) +
	( ((int)(news0*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.2 + 1.05*hrange
        ) * mask0;
  } 
  else
  {
    float news1 = (MIN(s[1][i],smax[1])-smin[1]) / (smax[1]-smin[1]);    /* rescaled s[1] */
    int   mask1 = s[1][i]>=smin[1]-LATITUDE;                             /* s1 mask */

    if ( colour_type == 0 ) /* solid colours */
      bg[i] = A + newbg*hrange*0.95*(1-mask0)*(1-mask1) +
	( news0*hrange*0.9 + hrange*1.05 ) * mask0 +
	( news1*hrange*0.9 + hrange*2.05 ) * mask1;
    else
      bg[i] = A + newbg*hrange*0.95*(1-mask0)*(1-mask1) +
	( ((int)(news0*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.2 + 1.05*hrange
        ) * mask0*(1-mask1) +
	( ((int)(news1*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.2 + 2.05*hrange
        ) * mask1*(1-mask0) +
	( ((int)((news0+news1)*5*0.999)) * 0.2*hrange +
	  newbg*hrange*0.5*0.4 + 3.05*hrange
        ) * mask0*mask1;
  }
}

/* }}} */
  /* {{{ output */

im.i=bg;

if ( output_type == 1 ) /* outputting ints instead of floats */
{
  ibg=malloc(size*sizeof(short int));
  for(i=0;i<size;i++)
    ibg[i]=bg[i];
  im.i=(float*)ibg;
  im.dt=DT_SIGNED_SHORT;
  im.bpv=sizeof(short int);
}

if (argc>10)
  avw_write(argv[12],im);
else
  avw_write(argv[9],im);

/* }}} */

  return(0);
}

/* }}} */
